// This silly little Fiddler extension flips all GIF/PNG/JPG images upside-down as they flow across the wire.
// It introduces a "Flip Images" item on the Tools menu that enables or disables the extension.
using System;
using System.IO;
using System.Drawing;
using System.Text;
using Fiddler;
using System.Windows.Forms;

[assembly: Fiddler.RequiredVersion("2.1.0.4")]
namespace AustralianImages
{
    public class ImageFlipper: IAutoTamper  
    {
        private bool bEnabled = false;
        MenuItem oMenuItem = new MenuItem("F&lip Images");

        #region IAutoTamper Members
        public void OnBeforeReturningError(Session oSession) { /* no-op */}
        public void AutoTamperRequestAfter(Session oSession){/* no-op */}
        public void AutoTamperRequestBefore(Session oSession){/* no-op */}
        public void AutoTamperResponseAfter(Session oSession){/* no-op */}
        public void AutoTamperResponseBefore(Session oSession)
        {
            if (!bEnabled) return;

            if (null != oSession && oSession.responseCode == 200 && null != oSession.oResponse && null != oSession.oResponse.headers 
                && (oSession.oResponse.headers.ExistsAndContains("Content-Type", "image/gif") ||
                oSession.oResponse.headers.ExistsAndContains("Content-Type", "image/png") ||
                 oSession.oResponse.headers.ExistsAndContains("Content-Type", "image/jp"))
                ){
                try
                    {
                        oSession.utilDecodeResponse();
                        MemoryStream oStream = new MemoryStream(oSession.responseBodyBytes);
                        Bitmap oBMP = new Bitmap(oStream);
                        oBMP.RotateFlip(RotateFlipType.Rotate180FlipNone);
                        MemoryStream oNewStream = new MemoryStream();
                        oBMP.Save(oNewStream, System.Drawing.Imaging.ImageFormat.Png);
                        oSession.responseBodyBytes = oNewStream.GetBuffer();
                        oSession.oResponse.headers["Content-Length"] = oSession.responseBodyBytes.Length.ToString();
                        oSession.oResponse.headers["Content-Type"] = "image/png";
                        oSession.oResponse.headers["Cache-Control"] = "no-cache";
                        oStream.Dispose();
                        oNewStream.Dispose();
                    }
                    catch (Exception eX)
                    {
                        System.Diagnostics.Trace.WriteLine(eX.ToString(), "ImageFlipper Failed");
                    }
            }
        }
        #endregion

        #region IFiddlerExtension Members

        public void OnBeforeUnload()
        {
            bEnabled = false;
            oMenuItem.Dispose();
        }

        public void OnLoad()
        {
            oMenuItem.Click += delegate(object s, EventArgs e)
            {
                oMenuItem.Checked = !oMenuItem.Checked;
                bEnabled = oMenuItem.Checked;
            };
            FiddlerApplication.UI.mnuTools.MenuItems.Add(oMenuItem);
        }

        #endregion
    }
}
